
#ifdef __USAGE		/* palset.c */
Screen Blanker
%C [options] 

Options
   -d n   Delay before screen fade in seconds
   -D n   Delay before screen fade in minutes
   -f n   Length of fade in seconds
   -F n   Length of fade in tenths of seconds
   -u n   Length of un-fade in seconds
   -U n   Length of un-fade in tenths seconds
   -m n   Percent minimum brightness
   -k n   Number of key strokes to prevent immediate fading
   
Defaults: Delay 5min, Fade for 10 sec, Un-fade for 0 sec.

#endif
#include <stdio.h>
#include <string.h>
#include <conio.h>
#include <stdlib.h>
#include <ctype.h>
#include <unistd.h>
#include <i86.h>
#include <signal.h>
#include <sys/irqinfo.h>
#include <sys/kernel.h>
#include <sys/proxy.h>
#include <sys/types.h>
#include <sys/osinfo.h>
#include <sys/psinfo.h>
#include <sys/sched.h>
#include <stddef.h>


#define endif
#define endfor
#define endwhile
#define endswitch
#define forever (;;)
#define endforever

#define PalDegBits 16
#define MaxPalDeg (1l<<PalDegBits)

int PaletteValid;
int Impatient=0;
int ImpatientCount=0;
int verbose=0;
int OpMode=0;
int FadeSpeed;
int FadeDelay = 100;
int UFadeSpeed;
int UFadeDelay = 0;
int PalValues[768];

unsigned OpDelay = 3000;
unsigned TICKCount;
long PalDeg=MaxPalDeg;
long pPalDeg;
long MinPalDeg=0;
pid_t TICKproxy;

pid_t far KBinterrupt() {
	TICKCount = OpDelay;
	if (PalDeg != MaxPalDeg) {
		ImpatientCount++;
		if (UFadeDelay==0) {
			PalDeg = MaxPalDeg;
			return( TICKproxy );
		} endif
		OpMode = 1;
	} endif
	
	return( 0 );
} /* KBinterrupt */

pid_t far TICKinterrupt() {
	if (OpMode) {
		if ((PalDeg+=UFadeSpeed)>=MaxPalDeg) {
			PalDeg = MaxPalDeg;
			OpMode=0;
			if (ImpatientCount < Impatient) TICKCount=2;
		} endif
		return( TICKproxy );
	} endif
	if (TICKCount) {
		TICKCount--;
		if (TICKCount==0) {
			ImpatientCount=0;
			return( TICKproxy );
		} endif
	} endif
	if ((!TICKCount) && (PalDeg>MinPalDeg)) {
		if ((PalDeg-=FadeSpeed) < MinPalDeg) PalDeg = MinPalDeg;
		return( TICKproxy );
	} endif
	return( 0 );
} /* TICKinterrupt */

void ExitFunc() {
	if (pPalDeg==MaxPalDeg) return;
	TICKCount = 32000;
	PalDeg = MaxPalDeg;
	SetPalette();
} /* ExitFunc */

main( int argc, char *argv[] ){
	int opt;
	pid_t rpid;
	long v;

	while ((opt = getopt( argc, argv, "k:m:U:F:u:f:d:D:v" )) != -1 ) {
		switch (opt) {
			case 'k':
				Impatient=strtol( optarg, NULL, 0);
				break;
			case 'd':
				v = strtol( optarg, NULL, 0 );
				if (v>6400) {
					fprintf( stderr, "*** WARNING *** Maximum delay of 6400 seconds used.\n");
					v = 6400;										
				} endif
				OpDelay = v*20;
				break;
			case 'D':
				v = strtol( optarg, NULL, 0 );
				if (v>54) {
					fprintf( stderr, "*** WARNING *** Maximum delay of 54 minutes used.\n");
					v = 54;										
				} endif
				OpDelay = v*1200;
				break;
			case 'v':
				verbose = (verbose << 1)|1;
				break;
			case 'm':
				v = strtol( optarg, NULL, 0 );
				if (v>99) {
					fprintf( stderr, "*** WARNING *** Minimum brightness must be less then 99%.\n");
					v = 99;
				} endif
				if (v<0) {
					fprintf( stderr, "*** WARNING *** Minimum brightness must be greator or equal to 0%.\n");
					v = 0;
				} endif
				MinPalDeg = (MaxPalDeg*v)/100;
				break;
			case 'f':
				v = strtol( optarg, NULL, 0 );
				if (v>1500) {
					fprintf( stderr, "*** WARNING *** Maximum fade length of 1500 seconds used.\n");
					v = 1500;
				} endif
				FadeDelay = v*20;
				break;
			case 'F':
				v = strtol( optarg, NULL, 0 );
				if (v>30000) {
					fprintf( stderr, "*** WARNING *** Maximum fade length of 30000 tenths of seconds used.\n");
					v = 30000;
				} endif
				FadeDelay = v;
				break;
			case 'u':
				v = strtol( optarg, NULL, 0 );
				if (v>1500) {
					fprintf( stderr, "*** WARNING *** Maximum un-fade length of 1500 seconds used.\n");
					v = 1500;
				} endif
				UFadeDelay = v*20;
				break;
			case 'U':
				v = strtol( optarg, NULL, 0 );
				if (v>30000) {
					fprintf( stderr, "*** WARNING *** Maximum un-fade length of 30000 tenths seconds used.\n");
					v = 30000;
				} endif
				UFadeDelay = v;
				break;
		} endswitch 
	} endwhile
	
	FadeSpeed = MaxPalDeg/FadeDelay;
	UFadeSpeed = (UFadeDelay)?(MaxPalDeg/UFadeDelay):MaxPalDeg;
	if (verbose) fprintf( stderr, "Delay to blank %d  Fade length %d  Fade const %d\n", OpDelay, FadeDelay, FadeSpeed );
	
	if (verbose==0) {
		signal( SIGHUP, SIG_IGN );	
		if (signal( SIGINT, SIG_IGN ) != SIG_IGN) {
			if (fork()) exit( EXIT_SUCCESS ); 
		} endif
		
		close( 0 );
		close( 1 );
		close( 2 );
	} endif
	
	atexit( ExitFunc );
	signal( SIGQUIT, exit );
	signal( SIGILL, exit );
	signal( SIGKILL, exit );
	signal( SIGTERM, exit );
	signal( SIGSTOP, exit );
	
	setscheduler( 0, SCHED_RR, 12 );

/*	qnx_pflags( _PPF_SERVER, _PPF_SERVER, NULL, NULL);
*/
	
	TICKCount = OpDelay;

	if ((TICKproxy = qnx_proxy_attach( 0, 0, 0, -1 )) == -1 ) exit( EXIT_FAILURE );
	if ((qnx_hint_attach(  1, &KBinterrupt, FP_SEG( &TICKCount ) )) ==-1) exit( EXIT_FAILURE );
	if ((qnx_hint_attach(  -1, &TICKinterrupt, FP_SEG( &TICKCount ) )) == -1 ) exit( EXIT_FAILURE );
	
	pPalDeg = PalDeg;
	ReadPalette();
	for (;;) {
		if ((rpid = Receive( TICKproxy, 0, 0 )) == -1 ) continue;
		if (rpid == TICKproxy) {
			if (pPalDeg == MaxPalDeg) ReadPalette();
			if (pPalDeg != PalDeg) {
				pPalDeg = PalDeg;
				if (PaletteValid) SetPalette();
				else {
					TICKCount = OpDelay;
					PalDeg = MaxPalDeg;
					OpMode=0;
				} endif
			} endif
		} endif
	} endforever
} /* main */

void ReadPalette() {
	int i, q = 0xFF;
	int *pv = PalValues;

	outp( 0x3CE, 6 );
	if (inp( 0x3CF ) & 1) {
		PaletteValid=0;
		return;
	} endif
	outp( 0x3C7, 0 );
	for (i=0; (i<768); i++) q &= ( *pv++ = inp( 0x3C9 ) );
	PaletteValid = (q!=0xFF);
	if (verbose & 2) if (!PaletteValid) printf("Invalid Palette\n");
} /* ReadPalette */

void SetPalette() {
	int i;
	int *pv = PalValues;

	outp( 0x3C8, 0 );
	for (i=0; i<768; i++) outp( 0x3c9, ((*pv++) * PalDeg) >> PalDegBits );
} /* SetPalette */
